/*
 * @(#)77    1.1  test/src/ClientJsse.java, jsse.test, jsse1  5/9/00  11:45:28
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */
import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.Security;
import java.security.KeyStore;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSession;
import com.ibm.net.ssl.SSLContext;
import com.ibm.net.ssl.TrustManagerFactory;
import com.ibm.net.ssl.TrustManager;
import com.ibm.net.ssl.KeyManagerFactory;

/*
 * ServerJsse and ClientJsse
 *
 * Sample applications to demonstrate a simple client/server interaction using JSSE. They will
 * communicate using all enabled cipher suites.
 *
 * For simplicity, the ServerJsse and ClientJsse use the same KeyStore file called "testkeys"
 * protected by the password "passphrase".  In a real application, the client and server 
 * should each have their own KeyStore files.
 *
 * You can build these sample program classes by issuing the following command:
 *
 *     javac ServerJsse.java
 *     javac ClientJsse.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * Running the client/server application requires that the Server is started first and then the Client
 * application.
 *
 * To start the ServerJsse, execute the following command:
 *
 *     java ServerJsse
 *
 * The ServerJsse uses port 8050. Once ServerJsse has started it waits for the ClientJsse connection.
 *
 * To start the client, issue the following command:
 * 
 *     java ClientJsse %hostname(optional)%
 *
 * where %hostname% is the name of the host running SeverJsse.  The default is localhost.
 *
 */

public class ClientJsse {
    static int N = 50, L = 100, R = 2;

    static SSLContext sslContext;

    ClientJsse() {
        super();
    }


    // Open up the commonly shared KeyStore to obtain the Trusted Certificates.
    // KeyStore is of type "JKS".  Filename is "testkeys" and password is "passphrase".
    public static void initContext() {
        try {
            Security.addProvider(new com.ibm.jsse.IBMJSSEProvider());

            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(new FileInputStream("testkeys"), "passphrase".toCharArray());
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("IbmX509");
            kmf.init(ks, "passphrase".toCharArray());

            TrustManager[] tm;
            TrustManagerFactory tmf = TrustManagerFactory.getInstance("IbmX509");
            tmf.init(ks);
            tm = tmf.getTrustManagers();

            sslContext = SSLContext.getInstance("SSL");
            sslContext.init(kmf.getKeyManagers(), tm, null);
        } catch (Throwable t) {
            System.out.println("Failed to read testkeys file.");
            t.printStackTrace();
        }
    }


    /** Starts the ClientJsse application.
      * @param args the name of the host the ClientJsse should connect to
      */
    public static void main(String args[]) {

        int i, j, len, ci_nonanon;

        String host = "127.0.0.1";
        int port = 8050;

        if ((args != null) && (args.length > 0)) {
            host = args[0];
            i = host.indexOf(':');
            if (i != -1) {
                try {
                    port = Integer.parseInt(host.substring(i+1));
                    host = host.substring(0,i);
                } catch (Exception e) {
                    System.err.println("ClientJsse: wrong address format");
                    System.exit(1);
                }
            }
        }
        if (args.length > 1)
            N = Integer.parseInt(args[1]);
        if (args.length > 2)
            L = Integer.parseInt(args[2]);
        byte buffer[] = new byte[L];

        System.out.println("ClientJsse: started.");

        initContext();

        try {
            if (sslContext == null) {
                System.out.println ("ClientJsse:  SSL client context NOT created.");
                return;
            }

            System.out.println("ClientJsse: SSL server context created.");
            SSLSocketFactory factory = sslContext.getSocketFactory();
            String[] cipher_suites = sslContext.getSocketFactory().getSupportedCipherSuites();

            System.out.println("ClientJsse: enabled cipher suites");
            for (i=0, ci_nonanon=0; i<cipher_suites.length; i++) {
                if (cipher_suites[i].indexOf("_anon_") < 0) {
                    ci_nonanon++;
                }
                System.out.println("   " + cipher_suites[i]);
            }

            SSLSocket ssl_sock;
            OutputStream ostr;
            InputStream istr;
            long t;
            String[] ecs = new String[1];

            for (int csi = 0; csi<ci_nonanon; csi++) {
                // Remove anonymous cipher suites. TrustManager requires a personal certificate.
                ecs[0] = cipher_suites[csi];
                if (ecs[0].indexOf("_anon_") >= 0) {
                    continue;
                }
                for (int n = 0; n < R; n++) {
                    t = System.currentTimeMillis();
                    try {
                        ssl_sock = (SSLSocket) factory.createSocket(host, port);
                        ssl_sock.setEnabledCipherSuites(ecs);

                        ssl_sock.startHandshake();
                        SSLSession session = ssl_sock.getSession();
                        System.out.println("\nClientJsse: SSL connection established");
                        System.out.println("   cipher suite:       " + session.getCipherSuite());
                    } catch (Exception se) {
                        System.out.println("\nClientJsse: can't connect using: " +
                                           cipher_suites[csi] + "\n" + se);
                        break;
                    }

                    if (L > 0) {
                        istr = ssl_sock.getInputStream();
                        ostr = ssl_sock.getOutputStream();

                        for (j=0; j<N; j++) {
                            if ((j==N-1) && (n == R-1) && (csi == ci_nonanon - 1))
                                buffer[0] = (byte)-1;
                            ostr.write(buffer, 0, L);
                            for (len = 0;;) {
                                try {
                                    if ((i = istr.read(buffer, len, L-len)) == -1) {
                                        System.out.println("ClientJsse: connection dropped by partner.");
                                        istr.close();
                                        return;
                                    }
                                    // System.out.println("ClientJsse: " + i + " bytes received.");
                                    if ((len+=i) == L) break;
                                } catch (InterruptedIOException e) {
                                    // System.out.println("ClientJsse: timeout.\n");
                                }
                            }
                        }
                    }
                    System.out.println("Messages = " + N*2 + "; Time = " +
                                       (System.currentTimeMillis() - t));
                    ssl_sock.close();
                    System.out.println("\nClientJsse: SSL connection closed.");
                }
            }

            Thread.sleep(1500);

            // plain socket benchmark
            if (L > 0) {
                // Socket bench
                Socket sock = new Socket(host, port);
                System.out.println("\nClientJsse: Socket connection established.");
                istr = sock.getInputStream();
                ostr = sock.getOutputStream();
                t = System.currentTimeMillis();
                for (j=0; j<N; j++) {
                    ostr.write(buffer, 0, L);
                    for (len = 0;;) {
                        if ((i = istr.read(buffer, len, L-len)) == -1) {
                            System.out.println("ClientJsse: connection dropped by partner.");
                            return;
                        }
                        if ((len+=i) == L) break;
                    }
                }
                System.out.println("Messages = " + N*2 + "; Time = " +
                                   (System.currentTimeMillis() - t));
                sock.close();
                System.out.println("\nClientJsse: Socket connection closed.");
                Thread.sleep(1500);
            }
        } catch (Exception e) {
            System.out.println(e);
        }
        System.out.println("ClientJsse: terminated.");
    }
}

