/*
 * @(#)77    1.1  test/src/ClientPKCS12Jsse.java, jsse.test, jsse1  5/9/00  11:45:28
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */
import java.io.*;
import java.net.*;
import java.security.*;

import javax.net.ssl.*;
import com.ibm.net.ssl.*;

/*
 * ServerPKCS12Jsse and ClientPKCS12Jsse
 *
 * Sample applications to demonstrate a simple client/server interaction using JSSE. They will
 * communicate using all enabled cipher suites.
 *
 * For simplicity, the ServerPKCS12Jsse and ClientPKCS12Jsse use the same PKCS12
 * KeyStore file called "testkeys.pfx" protected by the password "passphrase".  In a real 
 * application, the client and server should each have their own KeyStore files.
 *
 * You can build these sample program classes by issuing the following command:
 *
 *     javac ServerPKCS12Jsse.java
 *     javac ClientPKCS12Jsse.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * Running the client/server application requires that the Server is started first and then the Client
 * application.
 *
 * To start the ServerPKCS12Jsse, execute the following command:
 *
 *     java ServerPKCS12Jsse
 *
 * The ServerPKCS12Jsse uses port 8050. Once ServerPKCS12Jsse has started it waits 
 * for the ClientPKCS12Jsse connection.
 *
 * To start the client, issue the following command:
 * 
 *     java ClientPKCS12Jsse %hostname(optional)%
 *
 * where %hostname% is the name of the host running SeverPKCS12Jsse.  The default is 
 * localhost.
 *
 */

public class ClientPKCS12Jsse {
   static int N = 50, L = 100, R = 2;

   ClientPKCS12Jsse() {
      super();
   }

   static SSLContext sslContext;

   // Open up the commonly shared KeyStore to obtain the Trusted and Personal Certificates
   // KeyStore is of type "PKCS12".  Filename is "testkeys.pfx" and password is "passphrase".
   private static void initContext()
   {
      try {
         Security.addProvider(new com.ibm.jsse.IBMJSSEProvider());

         sslContext = SSLContext.getInstance("SSL","IBMJSSE");
         sslContext.init("PKCS#12","testkeys.pfx","passphrase");

      } catch (Throwable t) {
         System.out.println("Failed to read testkeys.pfx file");
         t.printStackTrace();
      }
   }


/** Starts the ClientPKCS12Jsse application.
  * @param args the name of the host the ClientPKCS12Jsse should connect to
  */
   public static void main(String args[]) {

      int i, j, len, ci_nonanon;

      String host = "127.0.0.1";
      int port = 8050;

      if ((args != null) && (args.length > 0)) {
         host = args[0];
         i = host.indexOf(':');
         if (i != -1) {
            try {
               port = Integer.parseInt(host.substring(i+1));
               host = host.substring(0,i);
            } catch (Exception e) {
               System.err.println("ClientPKCS12Jsse: wrong address format");
               System.exit(1);
            }
         }
      }
      if (args.length > 1)
         N = Integer.parseInt(args[1]);
      if (args.length > 2)
         L = Integer.parseInt(args[2]);
      byte buffer[] = new byte[L];

      System.out.println("ClientPKCS12Jsse: started.");

      initContext();

      try {
         if (sslContext == null) {
            System.out.println("ClientPKCS12Jsse: SSL client context NOT created.");
            return;
         }
         System.out.println("ClientPKCS12Jsse: SSL server context created.");
         SSLSocketFactory factory = sslContext.getSocketFactory();
         String[] cipher_suites = sslContext.getSocketFactory().getSupportedCipherSuites();

         System.out.println("ClientPKCS12Jsse: enabled cipher suites");
         for (i=0, ci_nonanon = 0; i<cipher_suites.length; i++) {
            if (cipher_suites[i].indexOf("_anon_") < 0) {
               ci_nonanon++;
            }
            System.out.println("   " + cipher_suites[i]);
         }


         SSLSocket ssl_sock;
         OutputStream ostr;
         InputStream istr;
         long t;
         String[] ecs = new String[1];

         for (int csi = 0; csi<ci_nonanon; csi++) {
            // Remove anonymous cipher suites.  TrustManager requires a personal certificate.
            ecs[0] = cipher_suites[csi];
            if (ecs[0].indexOf("_anon_") >= 0) {
               continue;
            }

            for (int n = 0; n < R; n++) {
               t = System.currentTimeMillis();
               try {
                  ssl_sock = (SSLSocket) factory.createSocket(host, port);
                  ssl_sock.setEnabledCipherSuites(ecs);

                  ssl_sock.startHandshake();
                  SSLSession session = ssl_sock.getSession();
                  System.out.println("\nClientPKCS12Jsse: SSL connection established");
                  System.out.println("   cipher suite:       " + session.getCipherSuite());
               } catch (Exception se) {
                  System.out.println("\nClientPKCS12Jsse: can't connect using: " +
                                     cipher_suites[csi] + "\n" + se);
                  se.printStackTrace();
                  break;
               }

               if (L > 0) {
                  istr = ssl_sock.getInputStream();
                  ostr = ssl_sock.getOutputStream();

                  for (j=0; j<N; j++) {
                     if ((j==N-1) && (n == R-1) && (csi == ci_nonanon - 1))
                        buffer[0] = (byte)-1;
                     ostr.write(buffer, 0, L);
                     for (len = 0;;) {
                        try {
                           if ((i = istr.read(buffer, len, L-len)) == -1) {
                              System.out.println("ClientPKCS12Jsse: connection dropped by partner.");
                              istr.close();
                              return;
                           }
                           // System.out.println("ClientPKCS12Jsse: " + i + " bytes received.");
                           if ((len+=i) == L) break;
                        } catch (InterruptedIOException e) {
                           // System.out.println("ClientPKCS12Jsse: timeout.\n");
                        }
                     }
                  }
               }
               System.out.println("Messages = " + N*2 + "; Time = " +
                                  (System.currentTimeMillis() - t));
               ssl_sock.close();
               System.out.println("\nClientPKCS12Jsse: SSL connection closed.");
            }
         }

         Thread.sleep(1500);

         // plain socket benchmark
         if (L > 0) {
            // Socket bench
            Socket sock = new Socket(host, port);
            System.out.println("\nClientPKCS12Jsse: Socket connection established.");
            istr = sock.getInputStream();
            ostr = sock.getOutputStream();
            t = System.currentTimeMillis();
            for (j=0; j<N; j++) {
               ostr.write(buffer, 0, L);
               for (len = 0;;) {
                  if ((i = istr.read(buffer, len, L-len)) == -1) {
                     System.out.println("ClientPKCS12Jsse: connection dropped by partner.");
                     return;
                  }
                  // System.out.println("ClientPKCS12Jsse: " + i + " bytes received.");
                  if ((len+=i) == L) break;
               }
            }
            System.out.println("Messages = " + N*2 + "; Time = " +
                               (System.currentTimeMillis() - t));
            sock.close();
            System.out.println("\nClientPKCS12Jsse: Socket connection closed.");
            Thread.sleep(1500);
         }
      } catch (Exception e) {
         System.out.println(e);
      }
      System.out.println("ClientPKCS12Jsse: terminated.");
   }
}
