/*
 * @(#)78    1.1  test/src/ServerPKCS12Jsse.java, jsse.test, jsse1  5/9/00  11:45:40
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */
import java.io.*;
import java.net.*;
import java.security.*;

import javax.net.ssl.*;
import com.ibm.net.ssl.*;

/*
 * ServerPKCS12Jsse and ClientPKCS12Jsse
 *
 * Sample applications to demonstrate a simple client/server interaction using JSSE. They will
 * communicate using all enabled cipher suites.
 *
 * For simplicity, the ServerPKCS12Jsse and ClientPKCS12Jsse use the same PKCS12
 * KeyStore file called "testkeys.pfx" protected by the password "passphrase".  In a real 
 * application, the client and server should each have their own KeyStore files.
 *
 * You can build these sample program classes by issuing the following command:
 *
 *     javac ServerPKCS12Jsse.java
 *     javac ClientPKCS12Jsse.java
 *
 * Make sure that the ibmjsse.jar file is in your CLASSPATH or extensions directory.
 *
 * Running the client/server application requires that the Server is started first and then the Client
 * application.
 *
 * To start the ServerPKCS12Jsse, execute the following command:
 *
 *     java ServerPKCS12Jsse
 *
 * The ServerPKCS12Jsse uses port 8050. Once ServerPKCS12Jsse has started it waits 
 * for the ClientPKCS12Jsse connection.
 *
 * To start the client, issue the following command:
 * 
 *     java ClientPKCS12Jsse %hostname(optional)%
 *
 * where %hostname% is the name of the host running SeverPKCS12Jsse.  The default is 
 * localhost.
 *
 */

public class ServerPKCS12Jsse {
   static int N = 50, L = 100, R = 2;

   ServerPKCS12Jsse() {
      super();
   }

   static SSLContext sslContext;

   // Open up the commonly shared KeyStore to obtain the Trusted and Personal Certificates
   // KeyStore is of type "PKCS12".  Filename is "testkeys.pfx" and password is "passphrase".
   private static void initContext()
   {
      try {
         Security.addProvider(new com.ibm.jsse.IBMJSSEProvider());

         sslContext = SSLContext.getInstance("SSL","IBMJSSE");
         sslContext.init("PKCS#12","testkeys.pfx", "passphrase");

      } catch (Throwable t) {
         System.out.println("Failed to read testkeys.pfx file.");
         t.printStackTrace();
         System.exit(1);
      }
   }

/** Starts the ServerPKCS12Jsse application.
  * @param args no parameters required
  */
   public static void main(String args[]) {
      int i, j, len;

      if (args.length > 0)
         N = Integer.parseInt(args[0]);
      if (args.length > 1)
         L = Integer.parseInt(args[1]);

      byte buffer[] = new byte[L];

      System.out.println("ServerPKCS12Jsse: started.");

      initContext();

      try {

         System.out.println("ServerPKCS12Jsse: SSL server context created.");
         System.out.println("ServerPKCS12Jsse: enabled cipher suites");
         String  enabled[] = sslContext.getSocketFactory().getSupportedCipherSuites();
         for (i=0; i < enabled.length; i++)
            System.out.println(enabled[i]);

         SSLServerSocketFactory factory = sslContext.getServerSocketFactory();
         SSLServerSocket ssl_server_sock = (SSLServerSocket)factory.createServerSocket(8050);
         SSLSocket ssl_sock; 
         InputStream istr;
         OutputStream ostr;
         long t;
         while (buffer[0] != -1) {
            for (int n=0; n<R; n++) {
               try {
                  ssl_sock = (SSLSocket)(ssl_server_sock.accept());
                  ssl_sock.setEnabledCipherSuites(enabled);

                  ssl_sock.startHandshake();
                  SSLSession session = ssl_sock.getSession();
                  System.out.println("\nServerPKCS12Jsse: SSL connection established");
                  System.out.println("   cipher suite:       " + session.getCipherSuite());
               } catch (IOException se) {
                  System.out.println("\nServerPKCS12Jsse: client connection refused\n" + se);
                  se.printStackTrace();
                  break;
               }

               if (L > 0) {
                  istr = ssl_sock.getInputStream();
                  ostr = ssl_sock.getOutputStream();

                  t = System.currentTimeMillis(); 
                  for (j=0;j<N;j++) {
                     for (len = 0;;) {
                        try {
                           if ((i = istr.read(buffer, len, L-len)) == -1) {
                              System.out.println("ServerPKCS12Jsse: connection dropped by partner.");
                              ssl_sock.close();
                              return;
                           }
                        } catch (InterruptedIOException e) {
                           System.out.println("waiting");
                           continue;
                        }
                        // System.out.println("ServerPKCS12Jsse: " + i + " bytes received.");
                        if ((len+=i) == L) break;
                     }
                     ostr.write(buffer, 0, L);
                  }
                  System.out.println("Messages = " + N*2 + "; Time = " +
                                     (System.currentTimeMillis() - t));
               }
               ssl_sock.close();
            }
         }
         ssl_server_sock.close();
         System.out.println("\nServerPKCS12Jsse: SSL connection closed.");
         Thread.sleep(1000);

         // plain socket benchmark
         if (L > 0) {
            // Socket bench
            ServerSocket server_sock = new ServerSocket(8050);
            Socket sock = server_sock.accept();
            System.out.println("\nServerPKCS12Jsse: Socket connection accepted.");
            istr = sock.getInputStream();
            ostr = sock.getOutputStream();

            t = System.currentTimeMillis();
            for (j=0;j<N;j++) {
               for (len = 0;;) {
                  if ((i = istr.read(buffer, len, L-len)) == -1) {
                     System.out.println("ServerPKCS12Jsse: connection dropped by partner.");
                     return;
                  }
                  // System.out.println("ServerPKCS12Jsse: " + i + " bytes received.");
                  if ((len+=i) == L) break;
               }
               ostr.write(buffer, 0, L);
            }
            System.out.println("Messages = " + N*2 + "; Time = " +
                               (System.currentTimeMillis() - t));
            sock.close();
            server_sock.close();
            System.out.println("\nServerPKCS12Jsse: Socket connection closed.");
            Thread.sleep(1000);
         }
      } catch (Exception e) {
         e.printStackTrace();
      }
      System.out.println("ServerPKCS12Jsse: terminated.");
   }
}
