/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */
 
//
//     SAMPLE IBMJCE CODE
//

/*
 * Usage: This sample code demonstrate how to generate a pair of
 *        RSA key (a public key and a private key) and use the key to
 *        digitally sign a message using the SHA1withRSA algorithm.
 *
 
 * Requirement: Usage of this sample file requires the installation of
 *        the IBMJCE provider on the running machine and the CLASSPATH
 *        points to the classes.
 *
 *
 * Running the program by typing the following in a command line:
 *
 *       java SampleSHA1withRSA KEYLENGTH
 *
 *       KEYLENGTH - the length of the key (in bits) to be generated/used.
 */


import java.io.*;
import java.security.PrivateKey;        
import java.security.PublicKey;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.Signature;


public class SampleRSASignatures {
    
    // message to be signed
    static final byte[] originalMessage = "testmessage_a very long test message".getBytes();


    /***************
     * MAIN PROGRAM
     ***************/
    public static void main(String argv[]) {
        
        KeyPairGenerator rsaKeyPairGen;     // the key pair generator
        KeyPair rsaKeyPair;                 // the key pair
        PublicKey rsaPub;                // the public key
        PrivateKey rsaPriv;              // the private key


    /*********************
     * Key Pair Generator
     *********************/
     
        System.out.println();
        
        
        // First, get an instance of a RSA key pair generator from the IBMJCE provider;
        System.out.println("Getting an instance of a RSA key pair generator from the IBMJCE provider");
        try {
            // "RSA" - the name of the algorithm
            // "IBMJCE" - the name of the provider
            rsaKeyPairGen = KeyPairGenerator.getInstance("RSA", "IBMJCE");
            
        } catch (Exception ex) {
            System.out.println("Error getting RSA Key Pair Generator!! : " + ex.getMessage());
            return;
        }
        System.out.println();


    /*****************************
     * Generate a pair of RSA key
     ****************************/
     
        // Then use the RSA key pair generator to generate a pair of keys.
        try {
            // Initialing the key pair generator is not necessary. This step
            // enables us to specify the length of the key to be generated. We 
            // can also specify a specific SecureRandom to be used when generating
            // the key.
            // We initialize the key pair generator.
            // The default key length for RSA is 512-bit.
            if (argv.length != 0) {
                int len = Integer.parseInt(argv[0]);
                System.out.println("Generating a pair of RSA keys of " + len + " key strength");    
            
                rsaKeyPairGen.initialize(len);
            
            } else {
                System.out.println("Generating a pair of RSA key of 1024 key strength");  
            }
            
            // Then, generate a key pair from the initialized key pair generator.
            rsaKeyPair = rsaKeyPairGen.generateKeyPair();
        } catch (Exception ex) {
            System.out.println("Error generating a pair of RSA key using Key Pair generator!! : " + ex.getMessage());
            return;
        }
        System.out.println();


     /*************************
      * Get the RSA public key
      *************************/
      
        // Now, get the public key from the RSA key pair.
        System.out.println("Obtain the RSA public key from the RSA key pair");
        try {
            // Using the getpublic method in the RSA key pair to obtain the 
            // RSA public key
            rsaPub = rsaKeyPair.getPublic();
            
            System.out.println(rsaPub);
            
        } catch (Exception ex) {
            System.out.println("Error obtaining the RSA public key: " + ex.getMessage());
            return;
        }
        System.out.println();
        
        
    /**************************
     * Get the RSA private key
     **************************/ 
     
        // Also, get the private key from the RSA key pair
        System.out.println("Obtain the RSA private key from the RSA key pair");
        try {
            // Using the getPrivate method in the RSA key pair to obtain the 
            // RSA private key
            rsaPriv = rsaKeyPair.getPrivate();
            
            System.out.println(rsaPriv);
            
        } catch (Exception ex) {
            System.out.println("Error obtaining the RSA private key: " + ex.getMessage());
            return;
        }
        System.out.println();
        
        
    /***************************************
     * Get a SHA1withRSA signature instance
     ***************************************/
     
        Signature sig = null;
        byte[] signature = null;
        
        // Now, get an instance of the SHA1iwithRSA signature
        try {
            System.out.println("Get a SHA1withRSA signature instance");
            sig = Signature.getInstance("SHA1withRSA", "IBMJCE");
        
        } catch (Exception ex) {
            System.out.println("Error getting SHA1withRSA instance: " + ex.getMessage());
        }
        System.out.println();
        
        
    /******************* 
     * Sign the message
     *******************/
     
        try { 
        
            /*
              1. Initialize (signing) with the private key  [Signature.initSign(PrivateKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Generate a signature                       [Signature.sign()]
             */
             
            System.out.println("    Signing a message");
        
            sig.initSign(rsaPriv);
            sig.update(originalMessage);
            signature = sig.sign();
        
        } catch (Exception ex) {
            System.out.println("Error signing the message: " + ex.getMessage());    
        }
        System.out.println();
        
        
     /*********************
      * Verify the message
      ********************/
        try {
            
             /*
              1. Initialize (verifying) with the public key [Signature.initVerify(PublicKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Verify with the signature                  [Signature.verify(byte[])]
             */
             
            System.out.println("    Verifying a message");
            System.out.println();
            
            sig.initVerify(rsaPub);
            sig.update(originalMessage);
            boolean result = sig.verify(signature);
            
            System.out.println("    Verification results: " + result);
            
        } catch (Exception ex) {
            System.out.println("Error verifying the signature: " + ex.getMessage());
        }
        System.out.println();
        
    
    /***************************************
     * Get a MD2withRSA signature instance
     ***************************************/
     
        sig = null;
        signature = null;
        
        // Now, get an instance of the MD2iwithRSA signature
        try {
            System.out.println("Get a MD2withRSA signature instance");
            sig = Signature.getInstance("MD2withRSA", "IBMJCE");
        
        } catch (Exception ex) {
            System.out.println("Error getting MD2withRSA instance: " + ex.getMessage());
        }
        System.out.println();
        
        
    /******************* 
     * Sign the message
     *******************/
     
        try { 
        
            /*
              1. Initialize (signing) with the private key  [Signature.initSign(PrivateKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Generate a signature                       [Signature.sign()]
             */
             
            System.out.println("    Signing a message");
        
            sig.initSign(rsaPriv);
            sig.update(originalMessage);
            signature = sig.sign();
        
        } catch (Exception ex) {
            System.out.println("Error signing the message: " + ex.getMessage());    
        }
        System.out.println();
        
        
     /*********************
      * Verify the message
      ********************/
        try {
            
             /*
              1. Initialize (verifying) with the public key [Signature.initVerify(PublicKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Verify with the signature                  [Signature.verify(byte[])]
             */
             
            System.out.println("    Verifying a message");
            System.out.println();
            
            sig.initVerify(rsaPub);
            sig.update(originalMessage);
            boolean result = sig.verify(signature);
            
            System.out.println("    Verification results: " + result);
            
        } catch (Exception ex) {
            System.out.println("Error verifying the signature: " + ex.getMessage());
        }
        System.out.println();


    /***************************************
     * Get a MD5withRSA signature instance
     ***************************************/
     
        sig = null;
        signature = null;
        
        // Now, get an instance of the MD5iwithRSA signature
        try {
            System.out.println("Get a MD5withRSA signature instance");
            sig = Signature.getInstance("MD5withRSA", "IBMJCE");
        
        } catch (Exception ex) {
            System.out.println("Error getting MD5withRSA instance: " + ex.getMessage());
        }
        System.out.println();
        
        
    /******************* 
     * Sign the message
     *******************/
     
        try { 
        
            /*
              1. Initialize (signing) with the private key  [Signature.initSign(PrivateKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Generate a signature                       [Signature.sign()]
             */
             
            System.out.println("    Signing a message");
        
            sig.initSign(rsaPriv);
            sig.update(originalMessage);
            signature = sig.sign();
        
        } catch (Exception ex) {
            System.out.println("Error signing the message: " + ex.getMessage());    
        }
        System.out.println();
        
        
     /*********************
      * Verify the message
      ********************/
        try {
            
             /*
              1. Initialize (verifying) with the public key [Signature.initVerify(PublicKey)]
              2. Update with the original message           [Signature.update(byte[])]    
              3. Verify with the signature                  [Signature.verify(byte[])]
             */
             
            System.out.println("    Verifying a message");
            System.out.println();
            
            sig.initVerify(rsaPub);
            sig.update(originalMessage);
            boolean result = sig.verify(signature);
            
            System.out.println("    Verification results: " + result);
            
        } catch (Exception ex) {
            System.out.println("Error verifying the signature: " + ex.getMessage());
        }
        System.out.println();

       
	}
}


