/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.security.PrivateKey;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import com.ibm.security.pkcs10.CertificationRequestInfo;
import com.ibm.security.pkcs10.CertificationRequest;
import com.ibm.security.pkcsutil.PKCSException;

/*
 * PKCS10CertReq
 *
 * Sample program to demonstrate how to create a CertificationRequest
 * object using the com.ibm.security.pkcs10.CertificationRequest class.
 *
 * This sample program uses the file creqinfo.der which is created by the sample
 * program PKCS10CertReqInfo.  Run the PKCS10CertReqInfo sample before running
 * this program.
 *
 * creqinfo.der
 * This file contains the DER-encoding of a PKCS #10 CertificationRequestInfo 
 * object generated by the PKCS10CertReqInfo sample program.
 *
 * This sample program also uses a file found in the samples directory:
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key. This key 
 * corresponds to the public key packaged in the CertificationRequestInfo
 * object.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS10CertReq.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS10CertReq %SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_PRI% is a private key file that is a companion to the public key
 * in the certificate that was used with the samples.pkcs10.PKCS10CertReqInfo program.
 */

public class PKCS10CertReq {

    public static void main(String[] args) {

        /*
         * Reconstruct the CertificationRequestInfo object that was created,
         * encoded and stored by the PKCS10CertReqInfo sample program.
         */
        String iFile = "creqinfo.der";
        System.out.println("Reading in encoded cert request info from file " + iFile);
        byte[] reqBytes = null;
        try {
            FileInputStream reqInfoFile = new FileInputStream(iFile);
            reqBytes = new byte[reqInfoFile.available()];
            reqInfoFile.read(reqBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate info file " + iFile);
            System.out.println("Have you run PKCS10CertReqInfo?");
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot access info file " + iFile);
            System.exit(1);
        }

        CertificationRequestInfo info = null;
        try {
            info = new CertificationRequestInfo(reqBytes);
        } catch (IOException e) {
            System.out.println("Error generating CertificationRequestInfo");
            System.exit(1);
        }

        /*
         * The private key to sign the request must correspond to the public
         * key that is sent in the request.
         */
        Properties sysProp = System.getProperties();                   
        String fileSep = sysProp.getProperty("file.separator");
        String kFile = ".." + fileSep + "sampleRSA.pri";
        if (args.length > 0) {
            kFile = args[0];
        }
        System.out.println("Reading a private key encoding from file " + kFile);
        FileInputStream privKeyFile = null;
        byte[] keyBytes = null;
        try {
            privKeyFile = new FileInputStream(kFile);
            keyBytes = new byte[privKeyFile.available()];
            privKeyFile.read(keyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot read the private key file " + kFile);
            System.exit(1);
        }

        /*
         * Obtain the RSA private key object corresponding to the public key
         * in the certificate request info by using a key factory and the 
         * DER-encoding of the private key.
         */

        /*
         * Get a factory for RSA keys.
         */
        System.out.println("Getting a factory for RSA keys");
        KeyFactory keyFactory = null;
        try {
            keyFactory = KeyFactory.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
            System.out.println("No factory for RSA keys");
            System.exit(1);
        }

        /*
         * Create a key specification from the DER-encoded private key.
         * RSA private keys are encoded according to the PKCS #8 standard.
         */
        System.out.println("Generating an RSA private key specification from the encoding in " + kFile);
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);

        /*
         * Use the factory to generate an RSA private key object from the
         * encoding stored in the private key flie.
         */
        System.out.println("Generating an RSA private key from its specification");
        PrivateKey privKey = null;
        try {
            privKey = keyFactory.generatePrivate(keySpec);
        } catch (InvalidKeySpecException e) {
            System.out.println("Invalid key specification for RSA private key");
            System.exit(1);
        }

        /*
         * Next, package the CertificationRequestInfo along with a signature
         * generated by the private key into a CertificationRequest object.
         * You must specify a digest algorithm to use in the signing operation.
         * Valid digest values are MD2, MD5, and SHA when using RSA keys, and 
         * SHA when using DSA keys.
         */
        System.out.println("Creating the certification request");
        String digestAlg = "MD5";
        CertificationRequest request = null;
        try { 
            request = new CertificationRequest(info, privKey, digestAlg);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("Digest algorithm " + digestAlg + " is not supported");
            System.exit(1);
        } catch (PKCSException e) {
            System.out.println("Cannot create the certification request");
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot create the certification request");
            System.exit(1);
        }

         /*
          * The resulting request is in standard PKCS #10 format and can be
          * sent to a Certificate Authority which can issue a certificate
          * based on the request information.
          */

         System.out.println("SUCCESS: Created certification request");
         System.out.println(request.toString()); 
    }
}
