/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.security.Principal;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import com.ibm.security.x509.X500Name;
import com.ibm.security.pkcsutil.PKCSAttribute;
import com.ibm.security.pkcsutil.PKCSAttributes;
import com.ibm.security.pkcsutil.PKCSOID;
import com.ibm.security.pkcs10.CertificationRequestInfo;

/*
 * PKCS10CertReqInfo
 *
 * Sample program to demonstrate how to create a CertificationRequestInfo
 * object using the com.ibm.security.pkcs10.CertificationRequestInfo class. 
 *
 * This sample program uses a file found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * You can build this sample program class by issuing the following command:
 *
 *      javac PKCS10CertReqInfo.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *      java PKCS10CertReqInfo %SAMPLE_RSA_CER%
 *
 * where %SAMPLE_RSA_CER% is a certificate file.
 */

public class PKCS10CertReqInfo {

    public static void main(String[] args) {

        /*
         * Create the subject's X.500 name.
         */
        X500Name name = null;
        try {
            name =
               new X500Name("CN=User 1, OU=IBM PKCS, O=IBM Corporation, C=US");
        } catch (IOException e) {
            System.out.println("Error creating subject name");
            System.exit(1);
        }

        /*
         * Obtain a public key to be packaged in the request info.  This key
         * may be generated by a key pair generator, for example.  For this
         * program, we will extract the public key from the sample certificate
         * file, sampleRSA.cer.
         */

        /*
         * The certificate file.
         */
        Properties sysProp = System.getProperties();                   
        String fileSep = sysProp.getProperty("file.separator");
        String cFile = ".." + fileSep + "sampleRSA.cer";
        if (args.length > 0) {
            cFile = args[0];
        }
        FileInputStream certFile = null;
        System.out.println("Certificate file: " + cFile);
        try {
            certFile = new FileInputStream(cFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the certificate file " + cFile);
            System.exit(1);
        }

        /*
         * Get a factory for X.509 certificates.
         */
        System.out.println("Getting a factory for X.509 certificates");
        CertificateFactory certFactory = null;
        try {
            certFactory = CertificateFactory.getInstance("X509");
        } catch (CertificateException e) {
            System.out.println("No factory for X.509 certificates");
            System.exit(1);
        }

        /*
         * Use the factory to generate a certificate object from an encoding
         * stored in the certificate flie.
         */
        System.out.println("Generating a certificate from the encoding in " + cFile);
        Certificate cert = null;
        try {
            cert = certFactory.generateCertificate(certFile);
        } catch (CertificateException e) {
            System.out.println("X.509 certificate parsing error");
            System.exit(1);
        }

        /*
         * Now get the public key from the certificate.
         */ 
        PublicKey publicKey = ((X509Certificate) cert).getPublicKey();

        /* 
         * Create an optional set of attributes.
         */
        PKCSAttribute[] attrs = new PKCSAttribute[1];
        try {
            attrs[0] = new PKCSAttribute(PKCSOID.EMAIL_ADDRESS_OID,
                                         "user1@us.ibm.com");
        } catch (IOException e) {
            System.out.println("Error creating attributes");
            System.exit(1);
        }
        PKCSAttributes attributes = new PKCSAttributes(attrs);

        /*
         * Create the request information object.
         * This can then be used to create a CertificationRequest object.
         */
        CertificationRequestInfo info = new CertificationRequestInfo(name,
                                                        publicKey, attributes);

        /*
         * Store the DER-encoded object for use with the PKCS9CertReq sample 
         * program.
         */
        String rFile = "creqinfo.der";
        FileOutputStream reqFile = null;
        System.out.println("Storing the CertificationRequestInfo object");
        try {
            reqFile = new FileOutputStream(rFile);
            info.encode(reqFile); 
        } catch (IOException e) {
            System.out.println("Error storing encoded request info");
            System.exit(1);
        }

        System.out.println("SUCCESS: The CertificationRequestInfo object was created, DER-encoded and stored in file " + rFile); 
    }
}
