/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.security.PrivateKey;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CRL;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import com.ibm.security.pkcs12.PKCS12PFX;
import com.ibm.security.pkcsutil.PKCSException;

/*
 * PKCS12Pkcs12PfxSample2
 *
 * Sample program to demonstrate how to create a PKCS #12 PFX object and then
 * extract information from it using the com.ibm.security.pkcs12.PKCS12PFX 
 * class.  This class is used to create PKCS #12 PFX objects that adhere to the
 * PKCS #12 standard, but may not be compatible with common PFX users such
 * as Web browsers. 
 *
 * This sample program creates a PKCS12PFX object that uses password-based 
 * data integrity and password-based data privacy. 
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS12Pkcs12PfxSample2.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *      java PKCS12Pkcs12PfxSample2 %SAMPLE_RSA_CER% %SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_CER% is a certificate file and %SAMPLE_RSA_PRI% is a private key
 * file that is a companion to the public key in the certificate.
 */

public class PKCS12Pkcs12PfxSample2 {

    public static void main(String[] args) {

        /***********************************************
         * Creating a PKCS12PFX
         ***********************************************/

        /*
         * You must have some personal data to package into a PFX.  This data 
         * may include certificates, private keys, encrypted private keys 
         * (also called shrouded private keys) or certificate revocation lists.
         * This sample program will use the certificate and private key 
         * encodings found in the samples directory as personal information
         * items.
         */

        /*
         * Create the certificate to store in the PFX.
         */
        Properties sysProp = System.getProperties();
        String fileSep = sysProp.getProperty("file.separator");
        String cFile = ".." + fileSep + "sampleRSA.cer";
       
        if (args.length > 0) {
            cFile = args[0];
        }
        FileInputStream certFile = null;
        System.out.println("Certificate file: " + cFile);
        try {
            certFile = new FileInputStream(cFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the certificate file " + cFile);
            System.exit(1);
        }

        /* 
         * Get a factory for X.509 certificates. 
         */
        System.out.println("Getting a factory for X.509 certificates");
        CertificateFactory certFactory = null;
        try {
            certFactory = CertificateFactory.getInstance("X509");
        } catch (CertificateException e) {
            System.out.println("No factory for X.509 certificates");
            System.exit(1);
        }

        /* 
         * Use the factory to generate a certificate object from an encoding
         * stored in the certificate flie.
         */ 
        System.out.println("Generating a certificate from the encoding in " + cFile);
        Certificate cert = null; 
        try {
            cert = certFactory.generateCertificate(certFile);
        } catch (CertificateException e) {
            System.out.println("X.509 certificate parsing error");
            System.exit(1);
        }

        /*
         * Create the private key to store in the PFX.
         */ 
        String kFile = ".." + fileSep + "sampleRSA.pri";
        
        if (args.length > 1) {
            kFile = args[1];
        }
        System.out.println("Reading a private key encoding from file " + kFile);
        FileInputStream privKeyFile = null;
        byte[] keyBytes = null;
        try {
            privKeyFile = new FileInputStream(kFile);
            keyBytes = new byte[privKeyFile.available()];
            privKeyFile.read(keyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot read the private key file " + kFile);
            System.exit(1);
        }

        /*
         * Get a factory for RSA keys.
         */
        System.out.println("Getting a factory for RSA keys");
        KeyFactory keyFactory = null;
        try {
            keyFactory = KeyFactory.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
            System.out.println("No factory for RSA keys");
            System.exit(1);
        }

        /*
         * Create a key specification from the DER-encoded private key.
         * RSA private keys are encoded according to the PKCS #8 standard.
         */
        System.out.println(
            "Generating an RSA private key specification from the encoding in " + kFile);
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);

        /*
         * Use the factory to generate an RSA private key object from the
         * encoding stored in the private key flie.
         */
        System.out.println("Generating an RSA private key from its specification");
        PrivateKey privKey = null;
        try {
            privKey = keyFactory.generatePrivate(keySpec);
        } catch (InvalidKeySpecException e) {
            System.out.println("Invalid key specification for RSA private key");
            System.exit(1);
        }

        /* 
         * Create an empty PKCS12PFX to which the data items will be added.
         */
        System.out.println("Creating an empty PKCS12PFX");
        PKCS12PFX pfx = new PKCS12PFX();
       
        /* 
         * Data in a PKCS12PFX need not have any attached attributes.  That is, 
         * neither friendly names nor local key ids are required as per the 
         * standard.  Data in PKCS12PFXs is packaged into a "bag" which is a 
         * bundling of the data and its attributes (if any), all optionally 
         * protected.
         
        /*
         * Add the personal information. Data privacy will be ensured through 
         * password protection.  Specify an encryption algorithm and a password
         * to protect the data with.  Valid values for encryption algorithms 
         * are 40BitRC2, 128BitRC2, 40BitRC4, 128BitRC4, 2KeyTripleDES and 
         * 3KeyTripleDES.  The algorithms and passwords for protecting the 
         * individual data items may differ. 
         */
        char[] password1 = { 'l', 'a', '9', '5', 'l', 'T' };
        char[] password2 = { 'b', '3', '6', 'r', 'u', 'T' };

        String algName = "128BitRC2";
        try {
            pfx.addBagWithPasswordPrivacy(cert, null, algName, password1);
        } catch (PKCSException e) {
            System.out.println("Error adding data to the PFX");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        } catch (IOException e2) {
            System.out.println("Error adding data to the PFX");
            System.exit(1);
        } catch (NoSuchAlgorithmException e3) {
            System.out.println("Algorithm " + algName + " is not supported");
            System.exit(1);
        }
       

        algName = "3KeyTripleDES";
        try {
            pfx.addBagWithPasswordPrivacy(privKey, null, algName, password2);
        } catch (PKCSException e) {
            System.out.println("Error adding data to the PFX");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        } catch (IOException e2) {
            System.out.println("Error adding data to the PFX");
            System.exit(1);
        } catch (NoSuchAlgorithmException e3) {
            System.out.println("Algorithm " + algName + " is not supported");
            System.exit(1);
        }

        /* 
         * Apply a MAC to ensure data integrity.  The password used in this
         * step can be different from the other passwords.
         */
        char[] password3 = { 'V', '5', '5', 'm', 'o', 'w' };

        System.out.println("Applying a MAC to the PFX");
        try {
            pfx.applyMac(password3);
        } catch (PKCSException e) {
            System.out.println("Error protecting PFX");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        } catch (IOException e2) {
            System.out.println("Error protecting PFX");
            System.exit(1);
        } catch (NoSuchAlgorithmException e3) {
            System.out.println("An HMAC algorithm is not supported");
            e3.printStackTrace();
            System.exit(1);
        }

        /*
         * The DER-encoded PKCS #12 PFX can now be saved to a file. 
         */
         String pFile = "user1samp2.pfx";
         System.out.println("Writing the PFX to file " + pFile);
         try {
            pfx.write(pFile, false);
            System.out.println("SUCCESS: PFX created, encoded and written to file " + pFile);
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("ERROR: Cannot write PFX to file");
            System.exit(1);
        }

        /***************************************************
         * Extracting information from the PFX
         ***************************************************/

        /*
         * We can construct a PFX from its DER-encoding. In this PFX, password
         * mode was used to protect data privacy and ensure data integrity.
         */

        FileInputStream pfxFile = null;
        System.out.println("PFX file: " + pFile);
        byte[] pfxBytes = null;
        try {
            pfxFile = new FileInputStream(pFile);
            pfxBytes = new byte[pfxFile.available()];
            pfxFile.read(pfxBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the PFX file " + pFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot access the PFX file " + pFile);
            System.exit(1);
        }

        PKCS12PFX pfx2 = null;
        try {
            pfx2 = new PKCS12PFX(pfxBytes);
        } catch (IOException e) {
            System.out.println("Cannot create PFX from DER-encoding"); 
            System.exit(1);
        }
       
        /*
         * Verify the MAC to be sure that the data has not been tampered with.
         * You must provide the same password that the PFX was protected
         * with.
         */
        System.out.println("Verifying the PFX MAC");
        try {
            if (pfx2.verifyMac(password3)) {
                System.out.println("The PFX data is verified");
            } else {
                System.out.println("The PFX data is not verified");
                /* Possible other actions. */
            }
        } catch (IOException e) {
            System.out.println("IOException verifying MAC");
        }

        /* 
         * Extract personal data from the PFX. Unprotected data and data 
         * protected with the input password will be retrieved.  If it is 
         * known that the data is protected with different passwords, 
         * additional get* method calls with the differing passwords should be 
         * made.  
         */

        /* 
         * Attempt to get certificates using a bogus password. 
         */
        char[] badPassword = password2; 
        System.out.println("Extracting certificates with bad password"); 
        Certificate[] certs = null; 
        try {
            certs = pfx2.getAllCertificates(badPassword, null, null);
            if ((certs == null) || (certs.length == 0)) {
                System.out.println("Got no certificates with bad password");
            } else {
                System.out.println("Got " + certs.length + " certificates with bad password");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting certificates from PFX");
            e.getRelatedException().printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting certificates from PFX");
        }
       
        /* 
         * Attempt to get certificates using a good password. 
         */
        System.out.println("Extracting certificates with good password"); 
        certs = null; 
        try {
            certs = pfx2.getAllCertificates(password1, null, null);
            if ((certs == null) || (certs.length == 0)) {
                System.out.println("Got no certificates with good password");
            } else {
                System.out.println("Got " + certs.length + " certificates with good password");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting certificates from PFX");
            e.getRelatedException().printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting certificates from PFX");
        }
        /* 
         * Attempt to get certificates using a good password. 
         */
        System.out.println("Extracting certificates with good password"); 
        PrivateKey[] keys = null; 
        try {
            keys = pfx2.getAllPrivateKeys(password2, null, null);
            if ((keys == null) || (keys.length == 0)) {
                System.out.println("Got no private keys");
            } else {
                System.out.println("Got " + keys.length + " private keys");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting private keys from PFX");
            e.getRelatedException().printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting private keys from PFX");
        }
    }
}
