/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.security.PrivateKey;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CRL;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import com.ibm.security.pkcs12.PKCS12PFX;
import com.ibm.security.pkcsutil.PKCSException;

/*
 * PKCS12Pkcs12PfxSample3
 *
 * Sample program to demonstrate how to create a PKCS #12 PFX object and then
 * extract information from it using the com.ibm.security.pkcs12.PKCS12PFX 
 * class.  This class is used to create PKCS #12 PFX objects that adhere to the
 * PKCS #12 standard, but may not be compatible with common PFX users such
 * as Web browsers. 
 *
 * This sample program creates a PKCS12PFX object that uses public key
 * data integrity and public key data privacy. 
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * rcvrRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.  
 *
 * rcvrRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file rcvrRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS12Pkcs12PfxSample3.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS12Pkcs12PfxSample3 %SAMPLE_RSA_CER% %SAMPLE_RSA_REC_CER% %SAMPLE_RSA_PRI% %SAMPLE_RSA_REC_PRI%
 *
 * where %SAMPLE_RSA_CER% is a certificate file, %SAMPLE_RSA_PRI% is a private key
 * file that is a companion to the public key in the certificate,
 * %SAMPLE_RSA_REC_CER% is a receiver certificate file and %SAMPLE_RSA_REC_PRI% is a receiver
 * private key file that is a companion to the public key in the receiver certificate.
 */

public class PKCS12Pkcs12PfxSample3 {

    public static void main(String[] args) {

        /***********************************************
         * Creating a PKCS12PFX
         ***********************************************/

        /*
         * You must have some personal data to package into a PFX.  This data 
         * may include certificates, private keys, encrypted private keys 
         * (also called shrouded private keys) or certificate revocation lists.
         * This sample program will use the certificate and private key 
         * encodings found in the samples directory as personal information
         * items.
         */

        /*
         * Create the certificate to store in the PFX.
         */
        Properties sysProp = System.getProperties();
        String fileSep = sysProp.getProperty("file.separator");
        String cFile = ".." + fileSep + "sampleRSA.cer";
        
        if (args.length > 0) {
            cFile = args[0];
        }
        FileInputStream certFile = null;
        System.out.println("Certificate file: " + cFile);
        try {
            certFile = new FileInputStream(cFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the certificate file " + cFile);
            System.exit(1);
        }

        /*
         * Also instantiate a receiver's certificate from a DER-encoding.
         * In a real scenario, the receiver would already have his certificate
         * and private key. 
         */
        cFile = ".." + fileSep + "rcvrRSA.cer";
        
        if (args.length > 1) {
            cFile = args[1];
        }
        FileInputStream rcvrFile = null;
        System.out.println("Certificate file: " + cFile);
        try {
            rcvrFile = new FileInputStream(cFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the certificate file " + cFile);
            System.exit(1);
        }

        /* 
         * Get a factory for X.509 certificates. 
         */
        System.out.println("Getting a factory for X.509 certificates");
        CertificateFactory certFactory = null;
        try {
            certFactory = CertificateFactory.getInstance("X509");
        } catch (CertificateException e) {
            System.out.println("No factory for X.509 certificates");
            System.exit(1);
        }

        /* 
         * Use the factory to generate certificate objects from the encodings
         * stored in the certificate files.
         */ 
        System.out.println("Generating certificates");
        Certificate cert = null; 
        Certificate rcvrCert = null; 
        try {
            cert = certFactory.generateCertificate(certFile);
            rcvrCert = certFactory.generateCertificate(rcvrFile);
        } catch (CertificateException e) {
            System.out.println("X.509 certificate parsing error.");
            System.exit(1);
        }

        /*
         * Create the private key to store in the PFX.
         */ 

        String kFile = ".." + fileSep + "sampleRSA.pri";
        
        if (args.length > 2) {
            kFile = args[2];
        }
        System.out.println("Reading a private key encoding from file " + kFile);
        FileInputStream privKeyFile = null;
        byte[] keyBytes = null;
        try {
            privKeyFile = new FileInputStream(kFile);
            keyBytes = new byte[privKeyFile.available()];
            privKeyFile.read(keyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot read the private key file " + kFile);
            System.exit(1);
        }

        /*
         * Also instantiate a receiver's private key from a DER-encoding.
         * In a real scenario, the receiver would already have his certificate
         * and private key. 
         */
        kFile = ".." + fileSep + "rcvrRSA.pri";
        
        if (args.length > 3) {
            kFile = args[3];
        }
        System.out.println("Reading a private key encoding from file " + kFile);
        FileInputStream rcvrKeyFile = null;
        byte[] rcvrKeyBytes = null;
        try {
            rcvrKeyFile = new FileInputStream(kFile);
            rcvrKeyBytes = new byte[rcvrKeyFile.available()];
            rcvrKeyFile.read(rcvrKeyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot read the private key file " + kFile);
            System.exit(1);
        }

        /*
         * Get a factory for RSA keys.
         */
        System.out.println("Getting a factory for RSA keys");
        KeyFactory keyFactory = null;
        try {
            keyFactory = KeyFactory.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
            System.out.println("No factory for RSA keys");
            System.exit(1);
        }

        /*
         * Create a key specification from the DER-encoded private key.
         * RSA private keys are encoded according to the PKCS #8 standard.
         */
        System.out.println("Generating an RSA private key specifications");
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);
        PKCS8EncodedKeySpec rcvrKeySpec = new PKCS8EncodedKeySpec(rcvrKeyBytes);

        /*
         * Use the factory to generate an RSA private key object from the
         * encoding stored in the private key flie.
         */
        System.out.println("Generating an RSA private key from its specification");
        PrivateKey privKey = null;
        PrivateKey rcvrKey = null;
        try {
            privKey = keyFactory.generatePrivate(keySpec);
            rcvrKey = keyFactory.generatePrivate(rcvrKeySpec);
        } catch (InvalidKeySpecException e) {
            System.out.println("Invalid key specification for RSA private key");
            System.exit(1);
        }

        /* 
         * Create an array of certificates.  The certificates in the array 
         * belong to each intended receiver of the private information. 
         */
        System.out.println("Creating array of receiver certificates");
        Certificate[] rcvrCerts = new Certificate[1];
        rcvrCerts[0] = rcvrCert;

        /* 
         * Create an empty PKCS12PFX to which the data items will be added.
         */
        System.out.println("Creating an empty PKCS12PFX");
        PKCS12PFX pfx = new PKCS12PFX();
       
        /* 
         * Data in a PKCS12PFX need not have any attached attributes.  That is, 
         * neither friendly names nor local key ids are required as per the 
         * standard.  Data in PKCS12PFXs is packaged into a "bag" which is a 
         * bundling of the data and its attributes (if any), all optionally 
         * protected.
         
        /*
         * Add the personal information. Data privacy will be ensured through 
         * public-key protection.  
         */
        try {
            pfx.addBagWithPubkeyPrivacy(cert, null, rcvrCerts);
            pfx.addBagWithPubkeyPrivacy(privKey, null, rcvrCerts);
        } catch (PKCSException e) {
            System.out.println("Error adding data to the PFX.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e2) {
            System.out.println("Error adding data to the PFX.");
            e2.printStackTrace();
            System.exit(1);
        }
       
        /* 
         * Ensure data integrity by applying a digital signature to the PFX.  
         * Specify a digest and encryption algorithm for the signature, as well
         * as the signer's certificate and private key.  Valid values for the 
         * encryption algorithm are RSA (when using RSA keys) and DSA (when 
         * using DSA keys).  Valid digest algorithms are SHA1, MD2 and MD5 
         * (when using RSA) and SHA1 (when using DSA).
         */
        System.out.println("Applying a digital signature to the PFX");
        try {
            /*
             * We are using the packaged certificate and private key to do the
             * signing, but they need not be the same.
             */
            pfx.applySignature("SHA1", "RSA", cert, privKey);
        } catch (PKCSException e) {
            System.out.println("Error protecting PFX.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e2) {
            System.out.println("Error protecting PFX.");
            e2.printStackTrace();
            System.exit(1);
        } catch (NoSuchAlgorithmException e3) {
            System.out.println("An HMAC algorithm is not supported");
            e3.printStackTrace();
            System.exit(1);
        }

        /*
         * The DER-encoded PKCS #12 PFX can now be saved to a file. 
         */
         String pFile = "user1samp3.pfx";
         System.out.println("Writing the PFX to file " + pFile);
         try {
            pfx.write(pFile, false);
            System.out.println("SUCCESS: PFX created, encoded and written to file " + pFile); 
        } catch (IOException e) {
            System.out.println("ERROR: Cannot write PFX to file.");
            e.printStackTrace();
            System.exit(1);
        }

        /***************************************************
         * Extracting information from the PFX 
         ***************************************************/

        /* 
         * We can construct a PFX from its DER-encoding. In this PFX, public
         * key mode was used to protect data privacy and ensure data integrity. 
         */

        FileInputStream pfxFile = null;
        System.out.println("PFX file: " + pFile);
        byte[] pfxBytes = null;
        try {
            pfxFile = new FileInputStream(pFile);
            pfxBytes = new byte[pfxFile.available()];
            pfxFile.read(pfxBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the PFX file " + pFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot access the PFX file " + pFile);
            e.printStackTrace();
            System.exit(1);
        }

        PKCS12PFX pfx2 = null;
        try {
            pfx2 = new PKCS12PFX(pfxBytes);
        } catch (IOException e) {
            System.out.println("Cannot create PFX from DER-encoding."); 
            e.printStackTrace();
            System.exit(1);
        }
       
        /*
         * Verify the digital signature to be sure that the data has not been 
         * tampered with.  You must use the public key corresponding to the 
         * private key that signed the PFX.  This public key is usually found
         * within a certificate.
         */
        System.out.println("Verifying the PFX digital signature");
        try {
            if (pfx2.verifySignature(cert)) {
                System.out.println("The PFX data is verified");
            } else {
                System.out.println("The PFX data is not verified");
                /* Possible other actions. */
            }
        } catch (PKCSException e) {
            System.out.println("Error verifying signature.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e2) {
            System.out.println("Unsupported signature algorithm.");
            e2.printStackTrace();
        } catch (IOException e3) {
            System.out.println("IOException verifying signature.");
            e3.printStackTrace();
        }

        /* 
         * Extract personal data from the PFX. Data accessible with the input 
         * certificate and private key will be retrieved.  If it is known that 
         * the data is accessible with different certificates and keys, 
         * additional get* method calls with the different arguments should be 
         * made.  The get* methods will not return unprotected data.
         */

        /* 
         * Attempt to get certificates using a bogus (ie, non-receiver)
         * certificate and private key.
         */
        System.out.println("Extracting certificates with bad certificate and private key"); 
        Certificate[] certs = null; 
        try {
            certs = pfx2.getAllCertificates(null, privKey, cert);
            if ((certs == null) || (certs.length == 0)) {
                System.out.println("Got no certificates with bad certificate and private key");
            } else {
                System.out.println(
                    "Got " + certs.length + " certificates with certificate and private key");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting certificates from PFX.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting certificates from PFX.");
            e2.printStackTrace();
        }
       
        /* 
         * Attempt to get certificates using a good certificate and private key.
         */
        System.out.println("Extracting certificates with good certificate and private key"); 
        certs = null; 
        try {
            certs = pfx2.getAllCertificates(null, rcvrKey, rcvrCert);
            if ((certs == null) || (certs.length == 0)) {
                System.out.println("Got no certificates with good certificate and private key");
            } else {
                System.out.println(
                    "Got " + certs.length + " certificates with good certificate and private key");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting certificates from PFX.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting certificates from PFX.");
            e2.printStackTrace();
        }

        /* 
         * Attempt to get certificates using a good certificate and private key.
         */
        System.out.println("Extracting certificates with good certificate and private key"); 
        PrivateKey[] keys = null; 
        try {
            keys = pfx2.getAllPrivateKeys(null, rcvrKey, rcvrCert);
            if ((keys == null) || (keys.length == 0)) {
                System.out.println("Got no private keys");
            } else {
                System.out.println("Got " + keys.length + " private keys");
            }
        } catch (PKCSException e) {
            System.out.println("Error extracting private keys from PFX.");
            System.out.println("Original Exception:");
            e.getRelatedException().printStackTrace();
            System.out.println("PKCSException:");
            e.printStackTrace();
        } catch (IOException e2) {
            System.out.println("Error extracting private keys from PFX.");
            e2.printStackTrace();
        }
    }
}
