/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */


import java.util.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.security.PrivateKey;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import com.ibm.security.pkcs1.PKCS1;
import com.ibm.security.pkcsutil.PKCSException;

/*
 * PKCS1EncryptDecrypt
 *
 * Sample program to demonstrate encryption and decryption operations using
 * the com.ibm.security.pkcs1.PKCS1 class.
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer 
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key. 
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in 
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator 
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS1EncryptDecrypt.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS1EncryptDecrypt %SAMPLE_RSA_CER% %SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_CER% is a certificate file and %SAMPLE_RSA_PRI% is a private key
 * file that is a companion to the public key in the certificate.
 * 
 */

public class PKCS1EncryptDecrypt {

    public static void main(String[] args) {

        /*
         * The certificate file.
         */
        Properties sysProp = System.getProperties();
        String fileSep = sysProp.getProperty("file.separator");
        String cFile = ".." + fileSep + "sampleRSA.cer";
        if (args.length > 0) {
            cFile = args[0];
        }
        FileInputStream certFile = null;
        System.out.println("Certificate file: " + cFile);
        try {
            certFile = new FileInputStream(cFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the certificate file " + cFile);
            System.exit(1);
        }

        /*
         * The private key file.
         */
        String kFile = ".." + fileSep + "sampleRSA.pri";
        if (args.length > 1) {
            kFile = args[1];
        }
        FileInputStream privKeyFile = null;
        System.out.println("Private key file: " + kFile);
        try {
            privKeyFile = new FileInputStream(kFile);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        }

        /*
         * The bytes to encrypt.
         */
        String plaintextString = "The quick brown fox jumps over the lazy dog.";
        byte[] plaintextBytes = null;
        try {
            plaintextBytes = plaintextString.getBytes("8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            plaintextBytes = plaintextString.getBytes();
        }

        System.out.println("Original text: " + plaintextString); 

        /*
         * Create a certificate object containing an RSA public key by using a
         * certificate factory and the DER-encoding of a certificate. 
         */

        /* 
         * Get a factory for X.509 certificates. 
         */
        System.out.println("Getting a factory for X.509 certificates");
        CertificateFactory certFactory = null;
        try {
            certFactory = CertificateFactory.getInstance("X509");
        } catch (CertificateException e) {
            System.out.println("No factory for X.509 certificates");
            System.exit(1);
        }

        /* 
         * Use the factory to generate a certificate object from an encoding
         * stored in the certificate flie.
         */
        System.out.println("Generating a certificate from the encoding in " + cFile);
        Certificate cert = null; 
        try {
            cert = certFactory.generateCertificate(certFile);
        } catch (CertificateException e) {
            System.out.println("X.509 certificate parsing error");
            System.exit(1);
        }

        /* 
         * Obtain the RSA private key object corresponding to the public key 
         * in the certificate by using a key factory and the DER-encoding of 
         * the private key.
         */

        /* 
         * Get a factory for RSA keys. 
         */
        System.out.println("Getting a factory for RSA keys");
        KeyFactory keyFactory = null;
        try {
            keyFactory = KeyFactory.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
            System.out.println("No factory for RSA keys");
            System.exit(1);
        }

        /*
         * Create a key specification from the DER-encoded private key.
         * RSA private keys are encoded according to the PKCS #8 standard.
         */
        System.out.println("Generating an RSA private key specification from the encoding in " + kFile);
        byte[] encodedPrivKey = null;
        try {
            encodedPrivKey = new byte[privKeyFile.available()];
            privKeyFile.read(encodedPrivKey);
        } catch (IOException e) {
            System.out.println("Cannot access the private key file " + kFile);
            System.exit(1);
        }
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(encodedPrivKey); 

        /* 
         * Use the factory to generate an RSA private key object from the 
         * encoding stored in the private key flie.
         */
        System.out.println("Generating an RSA private key from its specification");
        PrivateKey privKey = null; 
        try {
            privKey = keyFactory.generatePrivate(keySpec);
        } catch (InvalidKeySpecException e) {
            System.out.println("Invalid key specification for RSA private key");
            System.exit(1);
        }

        /*
         * Create a PKCS1 object.  Specify a digest algorithm as the third
         * argument; valid values are SHA, MD2 and MD5.  The other arguments
         * are ignored in this release.
         */
        PKCS1 pkcs1 = new PKCS1(null, null, "MD5", null);

        /*
         * Encrypt the plaintext bytes using the public key in the certificate.
         */
        System.out.println("Encrypting the original text with an RSA public key");
        byte[] ciphertextBytes = null;
        try {
            ciphertextBytes = pkcs1.encrypt(cert, plaintextBytes);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("RSA encryption is not supported");
            System.exit(1);
        } catch (PKCSException e2) {
            System.out.println("Error encrypting message");
            e2.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /* 
         * Create another PKCS1 object and decrypt the ciphertext using the
         * corresponding RSA private key.
         */
        PKCS1 anotherPkcs1 = new PKCS1(null, null, "MD5", null);
        byte[] decryptedBytes =  null;
        System.out.println("Decrypting the encrypted text with an RSA private key");
        try {
            decryptedBytes = anotherPkcs1.decrypt(privKey, 
                                                  ciphertextBytes);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("RSA encryption is not supported");
            System.exit(1);
        } catch (PKCSException e2) {
            System.out.println("Error encrypting message");
            e2.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /*
         * Verify the results.
         */
        String decryptedString = null;
        try {
            decryptedString = new String(decryptedBytes, "8859_1");
        } catch (UnsupportedEncodingException e) {
            System.out.println("Unsupported Encoding Expection: " + e);
            decryptedString = new String(decryptedBytes);
        }
        System.out.println("Decrypted text: " + decryptedString);

        if (plaintextString.equals(decryptedString)) {
            System.out.println("SUCCESS: Original text matches decrypted text");
        } else {
            System.out.println("ERROR: Original text does NOT match decrypted text");
        }
    }
}
