/*
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * IBM Confidential
 * OCO Source Materials
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 *
 * (C) Copyright IBM Corp. 1999, 2000 All Rights Reserved.
 *
 * The source code for this program is not published or otherwise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * ===========================================================================
 */

import java.util.*;
import com.ibm.security.pkcs7.*;
import com.ibm.security.x509.*;
import com.ibm.security.util.*;
import com.ibm.security.pkcsutil.*;
import com.ibm.misc.*;
import java.security.cert.*;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.KeyFactory;
import javax.crypto.*;
import java.io.*;

/*
 * PKCS7EnvelopedData
 *
 * Sample program to demonstrate how to create a PKCS #7 EnvelopedData object
 * by acting as a sender in encrypting the contents and as a receiver in
 * decrypting the contents.  The specified certificate and private key are used
 * for both sending and receiving in this sample.
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS7EnvelopedData.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS7EnvelopedData -cert=%SAMPLE_RSA_CER% -privatekey=%SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_CER% is a certificate file and %SAMPLE_RSA_PRI% is a private key
 * file that is a companion to the public key in the certificate.
 */

public class PKCS7EnvelopedData {

    public static void main(String[] args) {

        try {

            String certfile = null;
            String privatekeyfile = null;
	    Properties sysProp = System.getProperties();                   
            String fileSep = sysProp.getProperty("file.separator");
            String defaultcertfile = ".." + fileSep + "sampleRSA.cer";
            String defaultprivatekeyfile = ".." + fileSep + "sampleRSA.pri";

            String cipherAlg = "DES";
            int keysize = -1;

            System.out.println("This test creates and decrypts an EnvelopedData object.");
            System.out.println(
                "Usage:  PKCS7EnvelopedData [-cipher={DES | DESede | RC2}] [-keysize=keySize]\r\n"
                + "\t[-cert=certificateFileName] [-privatekey=privateKeyFileName]");

            for (int i=0;i<args.length;i++) {
                String param = args[i].toLowerCase();
                boolean bfound = false;

                if (param.startsWith("-cipher=")) {
                    cipherAlg = args[i].substring(8);
                    bfound = true;
                }
                if (param.startsWith("-keysize=")) {
                    keysize = Integer.parseInt(args[i].substring(9));
                    bfound = true;
                }
                if (param.startsWith("-cert=")) {
                    certfile = args[i].substring(6);
                    bfound = true;
                }
                if (param.startsWith("-privatekey=")) {
                    privatekeyfile = args[i].substring(12);
                    bfound = true;
                }

                if (!bfound) {
                    System.out.println("ERROR:  Unknown parameter " + args[i] + ".");
                    System.exit(1);
                }
            }

            if ((!cipherAlg.equals("DES")) && ((!cipherAlg.equals("DESede"))
                && (!cipherAlg.equals("RC2")))) {
                System.out.println("ERROR:  Cipher Algorithm must be DES, DESede, or RC2.");
                System.exit(1);
            }

            if ((cipherAlg.equalsIgnoreCase("DES")) && (keysize == -1)) {
                keysize = 56;
            } else if ((cipherAlg.equalsIgnoreCase("DESede")) && (keysize == -1)) {
                keysize = 112;
            } else if ((cipherAlg.equalsIgnoreCase("RC2")) && (keysize == -1)) {
                keysize = 56;
            }

            if (certfile == null) {
                System.out.println("Using the default certificate file: " + defaultcertfile);
                certfile = defaultcertfile;
            }
            if (privatekeyfile == null) {
                System.out.println("Using the default private key file: "
                    + defaultprivatekeyfile);
                privatekeyfile = defaultprivatekeyfile;
            }

            // Read the certificate and private key DER-encoded files.

            System.out.println("Reading in certificate = " + certfile);
            System.out.println("Reading in private key file = " + privatekeyfile);

            FileInputStream fis = new FileInputStream((new File(certfile)).getAbsolutePath());
            CertificateFactory cf = CertificateFactory.getInstance("X509");
            X509Certificate cert = (X509Certificate) cf.generateCertificate(fis);
            fis.close();

            fis = new FileInputStream((new File(privatekeyfile)).getAbsolutePath());
            byte[] enckey = new byte[fis.available()];
            fis.read(enckey);
            fis.close();

            PKCS8EncodedKeySpec keyspec = new PKCS8EncodedKeySpec(enckey);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            PrivateKey privateKey = keyFactory.generatePrivate(keyspec);

            String msg = "Meet me at the bell tower.";
            byte[] encodedEnvData;

            // First, act as a sender and create the EnvelopedData object.

            System.out.println("**************** SENDER ****************");
            System.out.println("Cipher Algorithm: " + cipherAlg);
            System.out.println("Key size: " + keysize);
            System.out.println("Secret message: " + msg);

            // Create the data to be enveloped as a ContentInfo.
            System.out.println("Creating a ContentInfo object with content type Data.");
            Data data = new Data();
            try {
                data.setData(msg.getBytes("8859_1"));
            } catch (UnsupportedEncodingException e) {
                System.out.println("Unsupported Encoding Expection: " + e);
                data.setData(msg.getBytes());
            }
            ContentInfo cinfo = new ContentInfo(data);

            // Create the EnvelopedData.

            System.out.println("Encrypting the ContentInfo into an EnvelopedData object.");

            Certificate[] certs = new Certificate[1];
            certs[0] = cert;
            PKCSAttributes unprotectedAttrs = null;
            PKCSAttribute attr1 = null;
            unprotectedAttrs = new PKCSAttributes();
            attr1 = new PKCSAttribute(PKCSOID.UNSTRUCTURED_NAME_OID,
                "unstructuredName");
            unprotectedAttrs = unprotectedAttrs.addAttribute(attr1);

            EnvelopedData ed = new EnvelopedData(certs, cinfo, cipherAlg, keysize,
                unprotectedAttrs); 

            // Write out the EnvelopedData object, certificate, and private keys.

            String prefilename = "ed" + cipherAlg; 
            String edfilename = prefilename + ".p7m";
            byte[] bytes = null;
            FileOutputStream fout = null;

            System.out.println(
                "Writing ContentInfo wrapped EnvelopedData object to file " + edfilename);
            ContentInfo cienvelopeddata = new ContentInfo(ed);
            cienvelopeddata.write(edfilename, false);

            // Now, act as the recipient of the enveloped data.

            System.out.println("**************** RECIPIENT ****************");

            // Read in the enveloped data and other objects.

            System.out.println(
                "Reading ContentInfo wrapped EnvelopedData object from file " + edfilename);
            ContentInfo cienvelopeddata2 = new ContentInfo(edfilename, false);

            EnvelopedData ed2 = (EnvelopedData) cienvelopeddata2.getContent(); 

            System.out.println("Decrypting enveloped data.");

            ContentInfo decryptedCinfo = ed2.decrypt(privateKey, cert);

            // Extract the decrypted text.

            System.out.println("Extracting message.");
            Content content = decryptedCinfo.getContent();
            Data contentData = (Data) content;
            byte[] decryptedBytes = contentData.getData();
            String decryptedMsg = null;
            try {
                decryptedMsg = new String(decryptedBytes, "8859_1");
            } catch (UnsupportedEncodingException e) {
                System.out.println("Unsupported Encoding Expection: " + e);
                decryptedMsg = new String(decryptedBytes);
            }
            System.out.println("Decrypted message: " + decryptedMsg);      

            if (msg.equals(decryptedMsg)) {
                System.out.println(
                    "*** SUCCESS:  Decrypted message matches the original message.");
            } else {
                System.out.println(
                    "*** ERROR:  Decrypted message DOES NOT MATCH the original message.");
            }

            PKCSAttribute attr2 = ed2.getUnprotectedAttribute(PKCSOID.UNSTRUCTURED_NAME_OID);
            if (attr2 != null) {
                if (attr1.equals(attr2)) {
                    System.out.println(
                        "*** SUCCESS:  Original UnstructuredName matches decoded UnstructuredName.");
                } else {
                    System.out.println(
                        "*** ERROR:  Original UnstructuredName DOES NOT MATCH decoded UnstructuredName.");
                }
            } else {
                System.out.println("*** ERROR:  Cannot get UnstructuredName attribute.");
            }

            System.out.print("EXIT:  PKCS7EnvelopedData");

        } catch (PKCSException e2) {
            System.out.println("\r\nERROR:  PKCSException: " + e2);
            e2.printStackTrace();
            Exception e3 = e2.getRelatedException();
            if (e3 != null) {
                System.out.println("\r\nOriginal PKCSException stack:");
                e3.printStackTrace();
            } else {
                System.out.println("\r\nNo Original PKCSException stack available");
            }
        } catch (Exception e) {
            System.out.println("\r\nERROR:  Exception");
            e.printStackTrace();
        }

    }
}
