/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.*;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.KeyFactory;
import java.security.cert.*;
import com.ibm.security.pkcs7.*;
import com.ibm.security.pkcs9.*;
import com.ibm.security.pkcsutil.*;
import com.ibm.security.x509.*;
import com.ibm.security.util.*;

/*
 * PKCS7SignedData
 *
 * Sample program to demonstrate how to create a PKCS #7 SignedData object
 * and verify the signed contents.  This program also shows how to detach the
 * contents from a SignedData object and verify the signature using a detached
 * message.
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS7SignedData.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *      java PKCS7SignedData -cert=%SAMPLE_RSA_CER% -privatekey=%SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_CER% is a certificate file and %SAMPLE_RSA_PRI% is a private key
 * file that is a companion to the public key in the certificate.
 */

public class PKCS7SignedData {

    public static void main(String args[]) {

        String dataString = "The quick brown fox jumps over the lazy dog.";
        String baddataString = "The quick brown fox jumps over the lazy dog!";

        String certfile = null;
        String privatekeyfile = null;
        Properties sysProp = System.getProperties();    
        String fileSep = sysProp.getProperty("file.separator");
        String defaultcertfile = ".." + fileSep + "sampleRSA.cer"; 
        String defaultprivatekeyfile = ".." + fileSep + "sampleRSA.pri";

        String signatureAlgorithm = "SHA1withRSA";

        System.out.println("Test program to test PKCS7 SignedData methods.");
        System.out.println(
            "Usage:  PKCS7SignedData [-cert=signingCertificateFileName] "
            + "[-privatekey=privateKeyFileName]");

        for (int i=0;i<args.length;i++) {
            String param = args[i].toLowerCase();
            boolean bfound = false;

            if (param.startsWith("-cert=")) {
                certfile = args[i].substring(6);
                bfound = true;
            }
            if (param.startsWith("-privatekey=")) {
                privatekeyfile = args[i].substring(12);
                bfound = true;
            }

            if (!bfound) {
                System.out.println("ERROR:  Unknown parameter " + args[i] + ".");
                System.exit(1);
            }
        }

        if (certfile == null) {
            System.out.println("Using the default certificate file: " + defaultcertfile);
            certfile = defaultcertfile;
        }
        if (privatekeyfile == null) {
            System.out.println("Using the default private key file: " + defaultprivatekeyfile);
            privatekeyfile = defaultprivatekeyfile;
        }

        try {
            // Read the certificate and private key DER-encoded files.

            System.out.println("Reading in certificate = " + certfile);
            System.out.println("Reading in private key file = " + privatekeyfile);

            FileInputStream fis = new FileInputStream((new File(certfile)).getAbsolutePath());
            CertificateFactory cf = CertificateFactory.getInstance("X509");
            X509Certificate cert = (X509Certificate) cf.generateCertificate(fis);
            fis.close();

            fis = new FileInputStream((new File(privatekeyfile)).getAbsolutePath());
            byte[] enckey = new byte[fis.available()];
            fis.read(enckey);
            fis.close();

            PKCS8EncodedKeySpec keyspec = new PKCS8EncodedKeySpec(enckey);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            PrivateKey privateKey = keyFactory.generatePrivate(keyspec);

            String certificateAlias = cert.getSubjectDN().getName();

            // Create initial values.

            System.out.println("Creating the Data object.");

            Data data = new Data();
            try {
                data.setData(dataString.getBytes("8859_1"));
            } catch (UnsupportedEncodingException e) {
                System.out.println("Unsupported Encoding Expection: " + e);
                data.setData(dataString.getBytes());
            }

            ContentInfo contentInfo = new ContentInfo(data);

            // Create the SignedData object.

            Certificate[] certs = new Certificate[1];
            certs[0] = cert;
            CRL[] crls = null;
            PKCSAttributes signedAttributes = null;
            PKCSAttributes unsignedAttributes = null;
            PrivateKey[] privateKeys = new PrivateKey[1];
            privateKeys[0] = privateKey;
            boolean signatureOnly = false;

            System.out.println(
                "*** Creating the SignedData object and signing the contents.");
            System.out.println("This will take a moment...");

            SignedData signeddata = new SignedData(certs, crls, contentInfo,
                signatureAlgorithm, privateKeys, signedAttributes, unsignedAttributes,
                signatureOnly);

            SignerInfo signerinfo = signeddata.getSignerInfo(cert);

            System.out.println(
                "*** Checking if SignedData/SignerInfo has the certificate.");

            if (signeddata.hasCertificate(cert)) {
                System.out.println("SUCCESS:  SignedData has certificate "
                    + certificateAlias);
            } else {
                System.out.println("ERROR:  SignedData does not have certificate "
                    + certificateAlias);
            }

            if (signerinfo.hasCertificate(cert)) {
                System.out.println("SUCCESS:  SignerInfo has certificate "
                    + certificateAlias);
            } else {
                System.out.println("ERROR:  SignerInfo does not have certificate "
                    + certificateAlias);
            }

            System.out.println("*** Verifying SignedData with the certificate.");

            if (signeddata.verify(cert)) {
                System.out.println(
                    "SUCCESS:  SignedData verified with public key from certificate "
                    + certificateAlias);
            } else {
                System.out.println(
                    "ERROR:  SignedData not verified with public key from certificate "
                    + certificateAlias);
            }

            // Remove the content.

            signeddata.removeContent();

            // This test allows us to check a detached signature.
            try {
                if (signeddata.verify(cert,dataString.getBytes("8859_1"))) {
                    System.out.println(
                        "SUCCESS:  SignedData verified with the data bytes and the public key from certificate "
                        + certificateAlias);
                } else {
                    System.out.println(
                        "ERROR:  SignedData not verified with the data bytes and the public key from certificate "
                        + certificateAlias);
                }

                if (!signeddata.verify(cert,baddataString.getBytes("8859_1"))) {
                    System.out.println(
                        "SUCCESS:  SignedData did not verify with the bad data bytes and the public key from certificate "
                        + certificateAlias);
                } else {
                    System.out.println(
                        "ERROR:  SignedData VERIFIED with the BAD data bytes and the public key from certificate "
                        + certificateAlias);
                }
            } catch (UnsupportedEncodingException e) {
                System.out.println("ERROR: Unsupported Encoding Expection: " + e);
            }

            // writing the signedddata contents to a file

            String filename = "sd1.p7m";
            System.out.println("\r\nWriting ContentInfo wrapped SignedData object to file "
                + filename);
            System.out.println(filename
                + " Contains a DER-encoded SignedData object with a contents.");

            ContentInfo cisigneddata = new ContentInfo(signeddata);
            cisigneddata.write(filename, false);

            System.out.print("EXIT:  PKCS7SignedData");

        } catch (PKCSException e2) {
            System.out.println("\r\nERROR:  PKCSException: " + e2);
            e2.printStackTrace();
            Exception e3 = e2.getRelatedException();
            if (e3 != null) {
                System.out.println("\r\nOriginal PKCSException stack:");
                e3.printStackTrace();
            } else {
                System.out.println("\r\nNo Original PKCSException stack available");
            }
        } catch (Exception e) {
            System.out.println("\r\nERROR:  Exception");
            e.printStackTrace();
        }
    }

}
