/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.io.FileInputStream; 
import java.io.FileNotFoundException; 
import java.io.IOException; 
import java.security.PrivateKey;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import com.ibm.security.pkcsutil.PKCSException;
import com.ibm.security.pkcs8.PrivateKeyInfo;
import com.ibm.security.pkcs8.EncryptedPrivateKeyInfo;

/*
 * PKCS8EncrPrivKeyInfo
 *
 * Sample program to demonstrate the creation and manipulation of encrypted
 * private keys using the com.ibm.security.pkcs8.EncryptedPrivateKeyInfo class.
 *
 * This sample program uses the file encrkey.der which is created by the sample
 * program PKCS8PrivKeyInfo.  Run the PKCS8PrivKeyInfo sample before running 
 * this program. 
 *
 * encrkey.der 
 * This file contains the DER-encoding of an encrypted RSA private key. It is
 * generated by the PKCS8PrivKeyInfo sample program.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS8PrivKeyInfo.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS8EncrPrivKeyInfo
 */

public class PKCS8EncrPrivKeyInfo {

    public static void main(String[] args) {

        /*
         * One source of encrypted private keys are Web browser PKCS #12 PFX
         * files.  In this sample, we are using an encrypted private key
         * generated and stored DER-encoded into a file.
         */

        /*
         * Read in the encrypted key bytes.
         */
        String ekFile = "encrkey.der";
        System.out.println("Reading in encrypted key bytes from file " + ekFile);
        byte[] encryptedKeyBytes = null;
        try {
            FileInputStream encrKeyFile = new FileInputStream(ekFile);
            encryptedKeyBytes = new byte[encrKeyFile.available()];
            encrKeyFile.read(encryptedKeyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate encrypted key file " + ekFile);
            System.out.println("Have you run PKCS8PrivKeyInfo?");
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot access encrypted key file " + ekFile);
            System.exit(1);
        }
        
        /*
         * The same password used to encrypt the key must be used to decrypt
         * the key.
         */ 
        char[] passwordChars = { 'h', '0', '6', 'X', 'b', 'l', 'g' };
    
          
        /*
         * Create the EncryptedPrivateKeyInfo object from the encoding.
         */
        System.out.println("Creating an EncryptedPrivateKeyInfo object");
        EncryptedPrivateKeyInfo encryptedKeyInfo = null;
        try {
            encryptedKeyInfo = new EncryptedPrivateKeyInfo(encryptedKeyBytes);
        } catch (IOException e) {
            System.out.println("Error decoding encrypted private key");
            System.exit(1);
        }

        /*
         * Decrypt the encrypted private key with the protecting password.
         */
        System.out.println("Decrypting the encrypted private key");
        byte[] keyBytes = null;
        try { 
            keyBytes = encryptedKeyInfo.decrypt(passwordChars);
        } catch (IOException e) {
            System.out.println("Error decrypting encrypted private key");
            System.exit(1);
        } catch (PKCSException e) {
            System.out.println("Error decrypting encrypted private key");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /*
         * Now create a PrivateKeyInfo object from the decrypted key bytes.
         */
        System.out.println("Creating a PrivateKeyInfo object from the decrypted key bytes");
        PrivateKeyInfo keyInfo = null;
        try {
            keyInfo = new PrivateKeyInfo(keyBytes);
        } catch (IOException e) {
            System.out.println("Error decoding private key");
            System.exit(1);
        }

        /*
         * Get the private key's algorithm name and a key factory for that 
         * algorithm.
         */
        String algName = keyInfo.getAlgorithm();
        System.out.println("Getting a key factory for " + algName + " keys"); 
        KeyFactory keyFact = null;
        try {
            keyFact = KeyFactory.getInstance(algName);
        } catch (NoSuchAlgorithmException e) {
            System.out.println("No factory for " + algName + " keys");
            System.exit(1);
        }

        /*
         * A private key specification can also be created from the decrypted 
         * key bytes.
         */
        System.out.println("Generating a key spec from the decrypted key bytes");
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);

        /*
         * Use the key factory and key specification to create an instance of 
         * a real RSA private key object.
         */
        System.out.println("Creating a private key from the factory and key spec");
        PrivateKey key = null; 
        try{
            key = keyFact.generatePrivate(keySpec);
        } catch (InvalidKeySpecException e) {
            System.out.println("Invalid key specification for " + algName + " private key");
            System.exit(1);
        }

        System.out.println("SUCCESS creating the key\n\n" + key.toString());

        /*
         * This private key object can now be used in other Java operations 
         * included in the Java cryptography framework.
         */
    }
}
