/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*; 
import java.io.FileInputStream; 
import java.io.FileNotFoundException; 
import java.io.IOException; 
import com.ibm.security.pkcsutil.PKCSException;
import com.ibm.security.pkcs8.PrivateKeyInfo;
import com.ibm.security.pkcs8.EncryptedPrivateKeyInfo;

/*
 * PKCS8PrivKeyInfo
 *
 * Sample program to demonstrate the creation and manipulation of private keys 
 * using the com.ibm.security.pkcs8.PrivateKeyInfo class.
 *
 * This sample program uses a file found in the samples directory:
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of a RSA private key. 
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac PKCS8PrivKeyInfo.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java PKCS8PrivKeyInfo %SAMPLE_RSA_PRI%
 *
 * where %SAMPLE_RSA_PRI% is a private key file.
 */

public class PKCS8PrivKeyInfo {

    public static void main(String[] args) {

        /*
         *  DER encodings of RSA and DSA private keys are suitable for decoding
         *  into PrivateKeyInfo objects.
         */

        /*
         * Get the key bytes from the DER-encoded RSA private key stored in
         * sampleRSA.pri.
         */
        Properties sysProp = System.getProperties();                          
        String fileSep = sysProp.getProperty("file.separator");           
        String kFile = ".." + fileSep + "sampleRSA.pri";
        
        if (args.length > 0) {
            kFile = args[0];
        }
        System.out.println("Reading a private key encoding from file " + kFile);
        FileInputStream privKeyFile = null;
        byte[] keyBytes = null;
        try {
            privKeyFile = new FileInputStream(kFile);
            keyBytes = new byte[privKeyFile.available()];
            privKeyFile.read(keyBytes);
        } catch (FileNotFoundException e) {
            System.out.println("Cannot locate the private key file " + kFile);
            System.exit(1);
        } catch (IOException e) {
            System.out.println("Cannot read the private key file " + kFile);
            System.exit(1);
        }

        /*
         * Select a password to encrypt the key with.
         */
        char[] passwordChars = { 'h', '0', '6', 'X', 'b', 'l', 'g' };

        /*
         * Convert the key encoding into a PrivateKeyInfo object.
         */
        System.out.println("Converting the key encoding into a PrivateKeyInfo object");
        PrivateKeyInfo keyInfo = null; 
        try {
            keyInfo = new PrivateKeyInfo(keyBytes);
        } catch (IOException e) {
            System.out.println("Error decoding the private key.");
            System.exit(1);
        }

        /* 
         * Encrypt the private key using a chosen password, digest algorithm and
         * encryption algorithm.  
         *
         *                      Valid digest values     Valid encryption values
         * =====================================================================
         * PKCS #5 Algorithms   MD2, MD5, SHA           RC2, DES
         * PKCS #12 Algorithms  SHA                     40BitRC2, 128BitRC2,
         *                                              40BitRC4, 128BitRC4,
         *                                              2KeyTripleDES, 
         *                                              3KeyTripleDES
         */

        System.out.println(
            "Encrypting the private key into an EncryptedPrivateKeyInfo object");
        EncryptedPrivateKeyInfo encryptedKeyInfo = null;
        try {
            encryptedKeyInfo = new EncryptedPrivateKeyInfo(passwordChars, keyInfo,
                "MD5", "DES");
        } catch (IOException e) {
            System.out.println("Error encoding the private key.");
            System.exit(1);
        } catch (PKCSException e) {
            System.out.println("Error encrypting the private key.");
            e.getRelatedException().printStackTrace();
            System.exit(1);
        }

        /* 
         * The encrypted private key may be stored in a DER-encoded binary
         * file or PKCS #12 PFX file.
         */
        System.out.println(
            "Writing the DER-encoded encrypted private key to file encrkey.der");
        try {
            encryptedKeyInfo.write("encrkey.der", false);  
            System.out.println("SUCCESS: Encrypted key is in file encrkey.der");
        } catch (IOException e) {
            System.out.println("Error writing the encrypted DER-encoded private key."); 
            System.exit(1);
        }
    }
}
