/* 
 * %Z%%M%    %I%  %W%  %G%  %U%
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * IBM Security Software Development Kit, Java (tm) Technology Edition
 * 
 * (C) Copyright IBM Corp. 2000 All Rights Reserved.
 * 
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

import java.util.*;
import java.io.*;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Principal;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.cert.*;
import com.ibm.security.pkcs7.*;
import com.ibm.security.pkcs8.*;
import com.ibm.security.pkcs10.*;
import com.ibm.security.smime.*;
import com.ibm.security.pkcsutil.*;
import com.ibm.security.x509.*;

/*
 * SMIMEReceive
 *
 * Sample program to demonstrate how to create a SMIMEReceive object to read
 * S/MIME messages.  Note that the S/MIME package does not provide a full MIME
 * parser so not all S/MIME or MIME message can be parsed.
 *
 * For signed messages, this program will verify the signature.
 * For encrypted messages, this program will decrypt the contents with the specified
 * recipient certificate.  If the encrypted message also contains a signed message,
 * this program will verify the decrypted signature.
 * For certification requests, this program will verify the request signature.
 *
 * This sample program uses two files found in the samples directory:
 *
 * sampleRSA.cer
 * This file contains the DER-encoding of an X.509 certificate.  This
 * certificate was generated by a certificate authority and contains an
 * RSA public key.
 *
 * sampleRSA.pri
 * This file contains the DER-encoding of an RSA private key.  This key is
 * the counterpart of the public key contained in the encoded certificate in
 * file sampleRSA.cer.  The key pair was generated by a KeyPairGenerator
 * instance.
 *
 * You can build this sample program class by issuing the following command:
 *
 *     javac SMIMEReceive.java
 *
 * Make sure that the ibmpkcs.jar file is in your CLASSPATH or extensions directory.
 *
 * To run this sample program, execute the following command:
 *
 *     java SMIMEReceive  -file=%SMIME_FILE% -cert=%SAMPLE_RSA_CER%
 *          -privatekey=%SAMPLE_RSA_PRI%
 *
 * where %SMIME_FILE% is the file name of the S/MIME message to parse, %SAMPLE_RSA_CER% is
 * the recipient certificate file, and %SAMPLE_RSA_PRI% is the recipient private key file
 * that is a companion to the public key in the recipient certificate.
 */

public class SMIMEReceive {

    public static void main(String args[]) {

        String infile = null;
        String certfile = null;
        String privatekeyfile = null;
        Properties sysProp = System.getProperties();                          
        String fileSep = sysProp.getProperty("file.separator");           
        String defaultcertfile = ".." + fileSep + "sampleRSA.cer";
        String defaultprivatekeyfile = ".." + fileSep + "sampleRSA.pri";
        
        String password = null;
        String recipientcertfile = null;

        System.out.println("Test program to read a SMIME mail file.");
        System.out.println("Usage:  smimeread -file=inputfilename\r\n"
            + "\t[-cert=senderCertFilename] [-recipientcert=recipientCertFilename]\r\n"
            + "\t[-privatekey=recipientPrivateKeyFilename | [-password=recipientPrivateKeyPassword]]");

        try {
            for (int i=0;i<args.length;i++) {
                String param = args[i].toLowerCase();
                boolean bfound = false;

                if (param.startsWith("-file=")) {
                    infile = args[i].substring(6);
                    bfound = true;
                }
                if (param.startsWith("-recipientcert=")) {
                    recipientcertfile = args[i].substring(15);
                    bfound = true;
                }
                if (param.startsWith("-cert=")) {
                    certfile = args[i].substring(6);
                    bfound = true;
                }
                if (param.startsWith("-privatekey=")) {
                    privatekeyfile = args[i].substring(12);
                    bfound = true;
                }
                if (param.startsWith("-password=")) {
                    password = args[i].substring(10);
                    bfound = true;
                }

                if (!bfound) {
                    System.out.println("ERROR:  Unknown parameter " + args[i] + ".");
                    System.exit(1);
                }
            }

            if (infile == null) {
                System.out.println("ERROR:  input file must be specified.");
                System.exit(1);
            }

            System.out.println("**** Begin Program Parameters ****");

            System.out.println("Input filename: " + infile);
            if ((certfile != null) && (privatekeyfile != null)) {
                System.out.println("\tSender Certificate Filename = " + certfile);
                System.out.println("\tPrivate Key Filename = " + privatekeyfile);
                System.out.println("\tPrivate Key Password = " + password);
            } else {
                System.out.println("Using the default certificate file: " + defaultcertfile);
                certfile = defaultcertfile;
                System.out.println("Using the default private key file: " + defaultprivatekeyfile);
                privatekeyfile = defaultprivatekeyfile;
            }
            if (recipientcertfile == null) {
                System.out.println(
                    "\tWARNING:  The recipient's certificate will default to the sender's certificate.");
            }
            System.out.println("Recipient Certificate Filename = " + recipientcertfile);

            System.out.println("**** End Program Parameters ****");

            // Read the certificates and private key DER-encoded files.

            System.out.println("Reading in certificate = " + certfile);

            X509Certificate cert = null;
            X509Certificate recipientcert = null;

            FileInputStream fis = new FileInputStream((new File(certfile)).getAbsolutePath());
            CertificateFactory cf = CertificateFactory.getInstance("X509");
            cert = (X509Certificate) cf.generateCertificate(fis);
            fis.close();

            PublicKey publicKey = cert.getPublicKey();

            if (recipientcertfile != null) {
                System.out.println("Reading in recipient certificate = "
                    + recipientcertfile);

                fis = new FileInputStream((new File(recipientcertfile)).getAbsolutePath());
                cf = CertificateFactory.getInstance("X509");
                recipientcert = (X509Certificate) cf.generateCertificate(fis);
                fis.close();
            } else {
                // Default the recipient cert to the sender cert.
                recipientcert = cert;
            }

            byte[] enckey = null;
            if (password != null) {
                System.out.println("Reading in encrypted private key file = " + privatekeyfile);
                EncryptedPrivateKeyInfo epki = new EncryptedPrivateKeyInfo(
                    privatekeyfile, false);
                enckey = epki.decrypt(password.toCharArray());
            } else {
                System.out.println("Reading in private key file = " + privatekeyfile);
                fis = new FileInputStream((new File(privatekeyfile)).getAbsolutePath());
                enckey = new byte[fis.available()];
                fis.read(enckey);
                fis.close();
            }

            PKCS8EncodedKeySpec keyspec = new PKCS8EncodedKeySpec(enckey);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            PrivateKey privateKey = keyFactory.generatePrivate(keyspec);

            // Read the S/MIME Message.

            System.out.println("Creating SMIMEReceiver object.");

            SMIMEReceiver smime = new SMIMEReceiver(infile);

            System.out.println("Output of SMIMEReceiver:\r\n" + smime);

            byte[] pkcs7bytes = smime.getContentInfoBytes();
            byte[] pkcs10bytes = smime.getCertificationRequestBytes();

            byte[] message = smime.getSMIMEMessage().getMessage();
            System.out.println("**** Begin Message ****");
            if (message != null) {
                try {
                    System.out.println(new String(message, "8859_1"));
                } catch (UnsupportedEncodingException e) {
                    System.out.println("Unsupported Encoding Expection: " + e);
                    System.out.println(new String(message));
                }
            } else {
                System.out.println("-NO MESSAGE-");
            }
            System.out.println("**** End Message ****");

            // Handle the p10 case.

            if (pkcs10bytes != null) {
                CertificationRequest cr = new CertificationRequest(pkcs10bytes);
                System.out.println("*** CertificationRequest ***\r\n"+cr);
                return;
            }

            if (pkcs7bytes == null) {
                System.out.println(
                    "\r\n*** Not a S/MIME Message or cannot be parsed by the SMIME package ***");
                return;
            }

            ContentInfo cinfo = new ContentInfo(pkcs7bytes);

            if (cinfo.isSignedData()) {
                System.out.println("*** ContentInfo is of SignedData type.");
                SignedData sd = (SignedData) cinfo.getContent();

                verifySignedData(sd, message, cert);

            } else if (cinfo.isEnvelopedData()) {
                if (privateKey == null) {
                    System.out.println(
                        "ERROR:  Private key must be specified to decrypt the EnvelopedData ContentInfo.");
                    return;
                }
                if (recipientcert == null) {
                    System.out.println(
                        "ERROR:  Recipient certificate must be specified to decrypt the EnvelopedData ContentInfo.");
                    return;
                }

                cinfo = cinfo.decrypt(privateKey, recipientcert);

                // We have a decrypted ContentInfo, which is a S/MIME message.
                // We want to view and verify the message contained within.

                if ((!cinfo.isData()) && (!cinfo.isSignedData())) {
                    System.out.println("*** Decrypted ContentInfo is of type " +
                        cinfo.getContentTypeString());
                } else if (cinfo.isSignedData()) {
                    // Decrypted contentInfo is of type SignedData, which means
                    // the sender sent a SignedData object that was enveloped.
                    // Extract the encapsulated content and display.

                    SignedData sd = (SignedData) cinfo.getContent();

                    byte[] messagecontained = sd.getEncapsulatedContentInfo().getContent();

                    System.out.println("Verifying message:\r\n"
                        + "***** Begin Message ****");
                    if (messagecontained != null) {
                        try {
                            System.out.println(new String(messagecontained, "8859_1"));
                        } catch (UnsupportedEncodingException e) {
                            System.out.println("Unsupported Encoding Expection: " + e);
                            System.out.println(new String(messagecontained));
                        }
                    } else {
                        System.out.println("null");
                    }
                    System.out.println("***** End Message ****\r\n");

                    verifySignedData(sd, messagecontained, cert);
                } else {
                    // Decrypted contentInfo is of type Data.
                    Data data = (Data) cinfo.getContent();

                    SMIMEMessage smimemessage = new SMIMEMessage(data.getData());

                    System.out.println(
                        "*** Contained SMIMEMessage:\r\n" + smimemessage);

                    ContentInfo cicontained = smimemessage.getContentInfo();

                    if ((cicontained != null) && (!cicontained.isSignedData())) {
                        System.out.println(
                            "*** Contained ContentInfo is of type "
                            + cicontained.getContentTypeString());
                    } else if (cicontained != null) {
                        SignedData sd = (SignedData) cicontained.getContent();

                        byte[] messagecontained = smimemessage.getMessage();

                        System.out.println("Verifying message:\r\n"
                            + "***** Begin Message ****");
                        if (messagecontained != null) {
                            try {
                                System.out.println(new String(messagecontained, "8859_1"));
                            } catch (UnsupportedEncodingException e) {
                                System.out.println("Unsupported Encoding Expection: " + e);
                                System.out.println(new String(messagecontained));
                            }
                        } else {
                            System.out.println("null");
                        }
                        System.out.println("***** End Message ****\r\n");

                        verifySignedData(sd, messagecontained, cert);
                    }

                }
            } else {
                System.out.println("ERROR:  Unsupported ContentInfo type = "
                    + cinfo.getContentTypeString());
            }

            System.out.print("EXIT:  SMIMEReceive");

        } catch (PKCSException e2) {
            System.out.println("\r\nERROR:  PKCSException: " + e2);
            e2.printStackTrace();
            Exception e3 = e2.getRelatedException();
            if (e3 != null) {
                System.out.println("\r\nOriginal PKCSException stack:");
                e3.printStackTrace();
            } else {
                System.out.println("\r\nNo Original PKCSException stack available");
            }
        } catch (Exception e) {
            System.out.println("\r\nERROR:  Exception");
            e.printStackTrace();
        }
    }

    private static void verifySignedData(SignedData sd, byte[] message,
        Certificate cert) throws NoSuchAlgorithmException, IOException,
            PKCSException {

        byte[] bytestobeverified = null;

        if (sd.hasCertificatesOnly()) {
            System.out.println("*** SignedData contains certificates only.");
            return;
        } else if (!sd.hasContent()) {
            System.out.println("*** SignedData contains a signature only.");
            if (message == null) {
                System.out.println("\tNo message to verify.");
                return;
            } else {
                System.out.println("\tVerifying message.");
                bytestobeverified = message;
            }
        } else {
            System.out.println("*** SignedData contains a signature and contents.");
        }

        if (cert != null) {
            System.out.println("*** Verifying with the specified certificate.");
            if (sd.verify(cert, bytestobeverified)) {
                System.out.println(
                    "SUCCESS:  Verified successfully with the specified certificate.");
                return;
            } else {
                System.out.println("Verification failed for the specified certificate.");
            }
        } else {
            System.out.println("*** Verifying with the SignedData certificates.");
            Certificate[] certs = sd.getCertificates();
            if (certs == null) {
                System.out.println("WARNING:  No certificates with which to verify the data.");
                return;
            }
            int numcerts = certs.length;

            for (int i=0;i<numcerts;i++) {
                cert = (X509Certificate) certs[i];
                System.out.println("Checking certificate["+i+"].");
                if (sd.verify(cert, bytestobeverified)) {
                    System.out.println(
                        "SUCCESS:  Verified successfully with certificate ["+i+"].");
                    return;
                } else {
                    System.out.println("Verification failed for certificate["+i+"].");
                }
            }
        }
        System.out.println("ERROR:  All verification failed.");
    }

}
